
#version 150
#define N 3


uniform sampler2D tex0;
uniform float dx;
uniform float dy;
uniform float gamma;
uniform bool invert;
uniform bool off;
uniform bool emboss;
uniform float alfa;
uniform float beta;
uniform int mappingActive;
uniform int edgeActive;
uniform float epsilon;
uniform float randomNumber;
uniform float k;
uniform float weight;


in vec2 texCoord;
out vec4 outputFrag;


float avg_red, avg_blue, avg_green;

float computeMaximum(vec3[N*N] window, int color){
    float max_value = 0.0, pixelColor, total=0.0f;
    for(int i=0; i<N*N; i++){
        switch (color) {
            case 0:
                pixelColor = window[i].r;
                break;
            case 1:
                pixelColor = window[i].g;
                break;
            case 2:
                pixelColor = window[i].b;
                break;
                
            default:
                break;
        }
        if(pixelColor > max_value){
            max_value = pixelColor;
        }
        total += pixelColor;
    }
    
    switch (color) {
        case 0:
            avg_red = total/(N*N);
            break;
        case 1:
            avg_green = total/(N*N);
            break;
        case 2:
            avg_blue = total/(N*N);
            break;
        default:
            break;
    }
    
    return max_value + epsilon;
}




//this method computes the brigthness of a pixel
float computeBrightness(vec4 color){
    //return (color.r+color.g+color.b)/3;
    return sqrt( 0.299*pow(color.r,2) + 0.587*pow(color.g,2) + 0.114*pow(color.b,2) );
}


//This method compute the entire window to apply the filter
vec3[N*N] computeWindow(vec2 texCoordinates){
    float deltaW = dx;
	float deltaH = dy;
    int i,j;
    
    
    vec3 window[N*N];
    for( i = 0; i < N; i++){
        for ( j = 0; j< N; j++) {
            window[i*N+j] = vec3(texture(tex0, vec2(texCoordinates.x -deltaW*floor(N/2) + deltaW*j, texCoordinates.y -deltaH*floor(N/2) + deltaH*i)));
        }
    }
    
	return window;
}


float mappingFunction(float colorIntensity){
    float returnValue;
    if(colorIntensity < alfa){
        returnValue =  0.0f;
    } else {
        returnValue = (colorIntensity - alfa)/(1.0f-alfa);
    }
    return returnValue;
}


void main(){
    vec3 window[N*N] = computeWindow(texCoord);
    float red, green, blue;
    
    float max_value_red = computeMaximum(window, 0);
    float max_value_green = computeMaximum(window, 1);
    float max_value_blue = computeMaximum(window, 2);
    float brigthness;
    
    if(mappingActive == 0){
        red = texture(tex0, vec2(texCoord.x, texCoord.y)).r/max_value_red;
        green = texture(tex0, vec2(texCoord.x, texCoord.y)).g/max_value_green;
        blue = texture(tex0, vec2(texCoord.x, texCoord.y)).b/max_value_blue;
    } else if (mappingActive == 1){
        
        red = mappingFunction(texture(tex0, vec2(texCoord.x, texCoord.y)).r/max_value_red);
        green = mappingFunction(texture(tex0, vec2(texCoord.x, texCoord.y)).g/max_value_green);
        blue = mappingFunction(texture(tex0, vec2(texCoord.x, texCoord.y)).b/max_value_blue);
        
    } else if(mappingActive == 2) {
        
        red = texture(tex0, vec2(texCoord.x, texCoord.y)).r/2.0f + avg_red/2.0f;
        red = mappingFunction(red/max_value_red);
        green = texture(tex0, vec2(texCoord.x, texCoord.y)).g/2.0f + avg_green/2.0f;
        green = mappingFunction(green/max_value_green);
        blue = texture(tex0, vec2(texCoord.x, texCoord.y)).b/2.0f + avg_blue/2.0f;
        blue = mappingFunction(blue/max_value_blue);
        
    }

    if(edgeActive == 1){
        brigthness = computeBrightness(texture(tex0, vec2(texCoord.x, texCoord.y)));
        if(brigthness < beta){
            red = 0.2f;
            green = 0.2f;
            blue = 0.2f;
        }
    } else if (edgeActive == 2){
        brigthness = computeBrightness(texture(tex0,vec2(texCoord.x, texCoord.y)));
        float t = k * brigthness;
        if(randomNumber <= t){
            red = red*weight + 1-weight;
            blue = blue*weight + 1-weight;
            green = green*weight + 1-weight;
        } else
        {
            red = 0.0f;
            blue = 0.0f;
            green = 0.0f;
        }
        
    }
    
    //outputFrag = vec4(computeBrightness(texture(tex0, vec2(texCoord.x, texCoord.y)))/max_value);
    outputFrag = vec4(red,green,blue,1.0);
    
}






