
#version 150
#define N 5


uniform sampler2D tex0;
uniform float dx;
uniform float dy;
uniform float alfa;
uniform float gamma;
uniform float beta;
uniform int edgeActive;
uniform float epsilon;


in vec2 texCoord;
out vec4 outputFrag;





//this method computes the function m(x,y) described in paper 6, equation (1)
float computeMaximum(float[N*N] window){
    float max_value = 0.0;
    for(int i=0; i<N*N; i++){
        if(window[i] > max_value){
            max_value = window[i];
        }
    }
    
    return max_value+epsilon;
}




//this method computes the brigthness of a pixel. We have found this formula in the internet
float computeBrightness(vec4 color){
    //return (color.r+color.g+color.b)/3;
    return sqrt( 0.299*pow(color.r,2) + 0.587*pow(color.g,2) + 0.114*pow(color.b,2) );
    //return (255 - color.r + 255 - color.g + 255 - color.b);
    //return 1.0;
    
}


//This method compute the entire window to apply the filter
float[N*N] computeWindow(vec2 texCoordinates){
    float deltaW = dx;
	float deltaH = dy;
    int i,j;
    
    float window[N*N];
    for( i = 0; i < N; i++){
        for ( j = 0; j< N; j++) {
            window[i*N+j] = computeBrightness(texture(tex0, vec2(texCoordinates.x -deltaW*floor(N/2) + deltaW*i, texCoordinates.y -deltaH*floor(N/2) + deltaH*j)));
        }
    }

	return window;
}


float mappingFunction(float colorIntensity){
    float returnValue;
    if(colorIntensity < alfa){
        returnValue =  0.0f;
    } else {
        returnValue = (colorIntensity - alfa)/(1.0f-alfa);
    }
    return returnValue;
}




void main(){
    vec3 color;
    float red,green, blue;
    float window[N*N] = computeWindow(texCoord);
    
    float max_value = computeMaximum(window);
    
    color = vec3(computeBrightness(texture(tex0, vec2(texCoord.x, texCoord.y))/max_value));
    if(edgeActive == 1){
        float brigthness = computeBrightness(texture(tex0, vec2(texCoord.x, texCoord.y)));
        if(brigthness < beta){
            red = 0.0f;
            green = 0.0f;
            blue = 0.0f;
            color = vec3(0.0,0.0,0.0);
        }
    }
    
    //outputFrag = vec4(computeBrightness(texture(tex0, vec2(texCoord.x, texCoord.y)))/max_value);
    
    outputFrag = vec4(color,1.0);
}





